/* conn_mgmt.c
*
* $Id: conn_mgmt.c 24577 2008-03-06 18:26:09Z stig $
* For connection management: keealive_routine(),hello_routine(),
  read_config_file(),write_config_file(),mmt_initialize(),mmt_cleanup()
* Wireshark - Network traffic analyzer
* By Aricent
* Copyright 1998 Gerald Combs
*
* Richard Sharpe, 13-Feb-1999, added support for initializing structures
*                              needed by dissect routines
* Jeff Foster,    2001/03/12,  added support tabbed hex display windowss
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
SPR Num:			Description:
	42				Appropriate error message through dialog boxes
					must be shown to user on failing to open connection mgmt
					port or message monitoring port at MMT.

	63				The option in MMT GUI to set auto capture mode on and off is not working correctly.
					If MMTSS goes down and comes up, the capturing must start automatically if autocapture
					mode is on but this is not happening.

	64				Connection management indicator is not working correctly and the GUI is hanging in case
					MMTSS is not sending any HELLO packets to MMT.

	66				MMT is not handling error conditions while reading from config file correctly.
*/
# ifdef HAVE_WINSOCK2_H 
  #include <windows.h>

#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <time.h>
#include <file.h>
#include <wiretap/file_util.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <gtk/filter_dlg.h>
#include <gtk/capture_dlg.h>
/* PMC I&V - 199 - start */
#include "menu.h"
/* PMC I&V - 199 - end */
#include "conn_mgmt.h"

#include "simple_dialog.h"

/* PMC MM#173 */
#include "main.h"

#define END_OF_LINE_STRING "\n"
#define CONFIG_FILE_KEY_VALUE_SEPARATOR "="

#define MIN_VALID_PORT_VALUE 1024
#define MAX_VALID_PORT_VALUE 65535

/*
 ** default IP address for MMTSS to use in case reading
 ** from config file fails.
 */

/* Commented For Autodetecting IP */
//#define DEF_MMT_IP_ADDR "127.0.0.1"

#define DEF_MMTSS_IP_ADDR "127.0.0.1"
/*
 ** This token gives the maximum length of the string that
 ** represents an IP address.
 */
/* PMC MM#445 */
//#define IP_ADDR_MAX_STR_LEN 16 
#define IP_ADDR_MAX_STR_LEN 15

/*
 ** First_hello_message_received is a global variable and its value is used to
 ** identify if the MMT-MMTSS connection is up or down. As soon as a hello
 ** message is recieved, the value of this variable is set to true. If no Hello
 **	message is recieved and maximu time to wait expires, this variable is set
 **	to FALSE again. Is accessed in menu.c to control autocapturing. Is modified
 ** in this file only by the hello_routine function
 */
gboolean first_hello_message_received = FALSE;

/*
 ** this file pointer points to the log file where all the log messages
 ** are dumped for MMT.
 */
static FILE	*fp_log;

/* PMC Support for absolute path to open file. IV-Sheet#253 - start */
static HMODULE wiresharkLib = NULL;
/* PMC Support for absolute path to open file. IV-Sheet#253 - end */

#define KEEP_ALIVE_MSG_SIZE 3

/*
 ** this variable contains the keepalive message that is sent to MMTSS
 ** with capture flag set to ON.
 */
static gchar gsend_message_capture_on[KEEP_ALIVE_MSG_SIZE];

/*  this variable contains the keepalive message that is sent to MMTSS
 ** with capture flag set to OFF
 */
static gchar gsend_message_capture_off[KEEP_ALIVE_MSG_SIZE];

/* Declare socket address structures for the message monitoring port */
static struct sockaddr_in gmmt_sock_addr_msg_monitor;

/* Declare socket address structures for connection mgmt port at MMT */
static struct sockaddr_in gmmt_sock_addr_conn_mgmt;

/* Declare socket address structures for the port at MMTSS */
static struct sockaddr_in gmmtss_sock_addr;

/* store length of the socket address for the message monitoring port */
unsigned int gmmt_sock_addr_msg_monitor_size;

/* store length of the socket address for the connection mgmt port at MMT */
unsigned int gmmt_sock_addr_conn_mgmt_size;

/* store length of the socket address for the port at MMTSS */
unsigned int gmmtss_sock_addr_size;

static SOCKET gmmt_sock_msg_monitor; /* Socket for message monitoring port */
static SOCKET gmmt_sock_conn_mgmt; /* Socket for connection mgmt port */
static SOCKET gmmtss_sock; /* Socket for the port at MMTSS */

/*
 ** These are the configuration parameters that shall be read in from
 ** the configuration file
 */
/* PMC MM#445 - start */

/*static guchar	*gmmtss_ipaddress; 
static guchar	*gmmt_ipaddress; */
/*MMTSS IP address*/
static guchar gmmtss_ipaddress[IP_ADDR_MAX_STR_LEN+FOR_NULLCHAR]; 
/*MMT IP address*/
static guchar gmmt_ipaddress[IP_ADDR_MAX_STR_LEN+FOR_NULLCHAR]; 

/* PMC MM#445 - end */

static guint32	gmmtss_port; /*MMTSS port number. by default 5003*/
static guint32	gmmt_port; /*MMT Message monitoring port. By default 5002*/
static guint32	gmmt_connmgmt_port; /*MMT Conn Mgmt port. by default 5004*/

/*
 ** This variable is used to store the time last hello message was received
 ** at MMT. This value shall be used to identify if maximum timer has expired
 ** since the last hello message was received or not.
 */
static time_t	g_time_last_hello_recvd_at = 0;

/*
 ** This is a temporary socket handle and is used by the hello_routine in an
 ** attempt to identify if the message monitoring port is up and occupied
 ** or not.
 */
static SOCKET	g_temp_socket; /*Socket for bind with MMT Message monitoring port */

#define BUFFER_SIZE_FOR_RECV_HELLO 30
/*
 ** This buffer is used to recive the hello message coming from
 ** MMTSS on MMT msg monitoring port
 */
static guchar g_buffer_for_hello_msg[BUFFER_SIZE_FOR_RECV_HELLO];

#define MMT_ERROR_STRING_MAX_SIZE 256
/*
 ** this character array is used to create error strings before they
 ** are dumped to MMT log file
 */
static char error_string[MMT_ERROR_STRING_MAX_SIZE + FOR_NULLCHAR];

/********************************************************************
* Function Name : mmt_log
*
* Description   : This function writes message into the log file.
*
* Returns       : None
**********************************************************************/
void mmt_log(char *message)
{ 
	time_t curr; /*variables to store current time*/
	struct tm *today;

	time(&curr); /* create a "timestamp" */
	today = localtime(&curr);
	if ( NULL != fp_log )
		fprintf(fp_log, "%02u:%02u:%02u %8s\n", today->tm_hour,
							today->tm_min, today->tm_sec,message);
	fflush(fp_log);
} 


/* PMC Autodetect IP Start */

/****************************************************
* FUNCTION     : inet_ntop2
*
* DISCRIPTION  : The inet_ntop() function shall convert a numeric address into
*                 a text string suitable for presentation.
*
* RETURN       : The return a pointer to the buffer containing the
*                 text string if the conversion succeeds, and NULL otherwise
******************************************************/

const char *inet_ntop2(int af, const void *src, char *dst, socklen_t cnt)
{
if (af == AF_INET)
{
	struct sockaddr_in in;
	memset(&in, 0, sizeof(in));
	in.sin_family = AF_INET;
	memcpy(&in.sin_addr, src, sizeof(struct in_addr));
	getnameinfo((struct sockaddr *)&in, sizeof(struct
	sockaddr_in), dst, cnt, NULL, 0, NI_NUMERICHOST);
	return dst;
}
else if (af == AF_INET6)
{
	struct sockaddr_in6 in;
	memset(&in, 0, sizeof(in));
	in.sin6_family = AF_INET6;
	memcpy(&in.sin6_addr, src, sizeof(struct in_addr6));
	getnameinfo((struct sockaddr *)&in, sizeof(struct
	sockaddr_in6), dst, cnt, NULL, 0, NI_NUMERICHOST);
	return dst;
}
	return NULL;
}

/*******************************************************
* Function      : getLocalIp
*
* Discription   : Return IP address of local machine.
*
* RETURN        : Pointer To a String Containing IP Address Of LocalHOst
*                  if succeeded otherwise NULL.
*******************************************************/

char *get_local_ip()
{


	int error;									/* Store Error no if any */
	char *ip=NULL;								/* Pointer To string containing IP adress */
	char strAddr[IP_ADDRLEN];																	
	struct addrinfo hints;						/* 
												** A pointer to an addrinfo
												** structure that provides hints
												** about the type of socket the 
												** caller supports
												*/
	struct addrinfo *result;					/* 
												** A pointer to a linked list of one or
												** more addrinfo structures that 
												** contains response information
												**  about the host
												*/
	struct sockaddr_in ipAddr4;					/* Structute For IPV4 address */
	WORD wVersionRequested;						/* for winsockdll intilization */
	WSADATA wsaData ;							/* for winsockdll intilization */
	int iResult ;								/* 
												** To store the return value of 
												** WSAStartUp 
												*/
	char  nodeName[MAX_NODE_LENGTH];			/* String to hold LocalHostName */
	int   nodeNameLength = MAX_NODE_LENGTH;		/* HostName Length */

	/* Currently used ver 1.1 */
	wVersionRequested = MAKEWORD(1, 1 );

	iResult = WSAStartup(wVersionRequested, &wsaData);
	if( iResult != 0 )
	{
		mmt_log("Error: WSAStartup() function call failed.");
		return NULL;
	}

	if( LOBYTE(wsaData.wVersion) != 1 || HIBYTE(wsaData.wVersion) != 1 )
	{   
		mmt_log("Error: winsock.dll version is different from 1.1");
		WSACleanup();
		return NULL;
	}

	/* Get Local Host Name */
	if( gethostname(nodeName,nodeNameLength))
	{
		mmt_log("Error: local host name couldn't be get.");
		return NULL;
	}
	memset( &hints, 0, sizeof(hints) ) ;
	hints.ai_family = AF_INET ;    
	error = getaddrinfo(nodeName, NULL, &hints, &result);   

	if (error)
	{
		mmt_log("Error: Couldn't get MMT IP address from local host name.");
		return NULL; 
	}

	memset( strAddr, 0, sizeof(strAddr));    
	memset( &ipAddr4, 0, sizeof(ipAddr4));

	ipAddr4.sin_family = AF_INET;

	memcpy( &ipAddr4, result->ai_addr,sizeof(ipAddr4));
	inet_ntop2( AF_INET, &ipAddr4.sin_addr, strAddr, sizeof(strAddr));

	/* Free memory occupied by Pointer */
	freeaddrinfo(result);

	/* releasing winsock dll */
	WSACleanup();

	/* copy the extracted ip address */

	ip = (char*) calloc ( IP_ADDRLEN, sizeof(char));
	memcpy ( ip, strAddr, IP_ADDRLEN);

	return (ip); 

}
/* PMC Autodetect IP End */

/* PMC I&V - 199 - start */
/************************************************************************
* Function Name  : is_blank_line
*
* Description    : This function checks for blank lines in file.
*
* Return         : int(TRUE/FALSE)
**************************************************************************/
int 
is_blank_line(char *line)
{
	int i = 0;
	int len;
	
	if( NULL == line )	
		return FALSE;
	
	 /* one less for new line character */
	len = strlen(line) - 1;

	/* check for spaces and tabs from beginning if some 
	nonspace (nontab) character is found return FALSE else return TRUE */
	for( i; i < len; i++ )
	{
		if( (line [i] != ' ') && (line[i] != '\t') ) 		
			return FALSE;		
	}
	return TRUE;
}
/* PMC I&V - 199 - end */
/************************************************************************
* Function Name  : read_config_file
*
* Description    : This function reads the configuration file.
*
* Returns        : None
**************************************************************************/
int read_config_file()
{
	/* This variable shall be used to read a line at a time from the config file*/
	gchar *line = (gchar *)ep_alloc(MAX_CONFIG_LINE_SIZE + FOR_NULLCHAR);
	FILE*  read_config_file_pointer = NULL; /*config file pointer*/
	guchar *key = NULL; /* tokens to read in the key */
	guchar *value = NULL; /* token value from the config file entry at each line*/
	guchar *ipaddr = NULL; /* Token to Store IP Address Returned by getLocalIp function*/

	/* PMC Support for absolute path to open file. IV Sheet#253 - start */
	char * file_name = NULL;
	/* PMC Support for absolute path to open file. IV Sheet#253 - end */

	/*counter to read in characters from the config file*/
	register guint32	char_counter;

	guint32 gretval; /*
					 ** variable to store and check the return value of
					 ** functions called from this function
					 */

	*line = '\0'; /*initialize line string*/

	/* PMC MM#445 - start */
	/*gmmt_ipaddress = (gchar *)ep_alloc(IP_ADDR_MAX_STR_LEN + FOR_NULLCHAR);
	gmmtss_ipaddress = (gchar *)ep_alloc(IP_ADDR_MAX_STR_LEN + FOR_NULLCHAR);
	g_snprintf(gmmtss_ipaddress, IP_ADDR_MAX_STR_LEN, DEF_MMTSS_IP_ADDR );*/
	memset(gmmt_ipaddress, 0, sizeof(gmmt_ipaddress));
	memset(gmmtss_ipaddress, 0, sizeof(gmmtss_ipaddress));
    /* PMC MM#445 - end */

	
	gmmtss_port=MMTSS_PORT;
	gmmt_port=MESSAGE_MONITORING_PORT;
	gmmt_connmgmt_port=CONN_MGMT_PORT;
	gdelay=SLEEP_TIMER;
	/**** Byte Alignment Changes Start****/

	/** It is assumed MMT runs on 
	 ** 64 bit machine by default */
	isDataFr64 = BIT64;

	/**** Byte Alignment Changes End****/

	if ( NULL == wiresharkLib )
	{
		wiresharkLib =  LoadLibrary(TEXT("libwireshark.dll"));
		if( NULL == wiresharkLib )
		{
			printf("\nconn_mgmt.c:mmt_initialize() Error in loading library");		
			return FAIL;		
		}
	}
	full_path = (char**) GetProcAddress(wiresharkLib, "full_path_name");
	file_name = (char *)calloc( strlen(*full_path) 
								+ strlen(CONFIG_FILE)
								+ FOR_NULLCHAR, sizeof(char) );
	if ( NULL == file_name )
	{
		mmt_log("Error(conn_mgmt.c:read_config_file): Memory allocation failed.");
		return FAIL;
	}
	sprintf( file_name, "%s%s", *full_path, CONFIG_FILE );
	read_config_file_pointer  = fopen(file_name,"r");

	free(file_name);
	file_name = NULL;
	
	if (read_config_file_pointer  == NULL)
	{	
		mmt_log("Error: Couldn't open configuration file for reading.");		
		return FAIL;
	}	
		
	ip_flag = 0;
	
	while ( NULL != fgets ( line, MAX_CONFIG_LINE_SIZE, read_config_file_pointer) ) 
	{   
		if( is_blank_line(line) )			 				
			continue;

		key = strtok(line, CONFIG_FILE_KEY_VALUE_SEPARATOR);
		if (NULL == key) {
			mmt_log("Error: Couldn't read key from configuration file");
			return FAIL;
		}
		else 
		{
			value = strtok(NULL, END_OF_LINE_STRING);
		}

		if ( NULL == value){
		        g_snprintf(error_string, MMT_ERROR_STRING_MAX_SIZE,
					"\"Error in Reading from configuration file :value is NULL for key \"%s\"\n"
					,key);
                mmt_log(error_string);
			return FAIL;
		}
		if ( 0 == strnicmp(key, "mmt_ip_address", strlen("mmt_ip_address")) )
		{
			g_snprintf(gmmt_ipaddress, IP_ADDR_MAX_STR_LEN, value );
			/* PMC MM#445 */
			/* gmmt_ipaddress[IP_ADDR_MAX_STR_LEN] = '\0';*/			
			ip_flag = 1;			
		}  
 		else if ( 0 == strnicmp(key, "mmt_msg_monitor_port",
            strlen("mmt_msg_monitor_port")) )
		{
			for ( char_counter=0;value[char_counter] != '\0';char_counter++)
			{
				gretval = isdigit(value[char_counter]);
				if (gretval == 0)
					break;
			}
			if (gretval != 0){
				gmmt_port = atoi(value);
			}
			else
				gmmt_port = MESSAGE_MONITORING_PORT;

		}
		else if ( 0 == strnicmp(key, "mmt_conn_mgmt_port",
                 strlen("mmt_conn_mgmt_port")) )
		{
			for ( char_counter=0;value[char_counter] != '\0';char_counter++)
			{
				gretval = isdigit(value[char_counter]);
				if (gretval == 0)
					break;
			}
			if (gretval != 0){
				gmmt_connmgmt_port = atoi(value);
			}
			else
				gmmt_connmgmt_port = CONN_MGMT_PORT;

		} else if ( 0 == strnicmp(key, "mmtss_ip_address",
                   strlen("mmtss_ip_address")) )
		{
			g_snprintf(gmmtss_ipaddress, IP_ADDR_MAX_STR_LEN, value );
			/* PMC MM#445 */
			/* gmmtss_ipaddress[IP_ADDR_MAX_STR_LEN] = '\0';*/			
		} 
		else if ( 0 == strnicmp(key, "mmtss_port", strlen("mmtss_port")) )
		{
			for ( char_counter=0;value[char_counter] != '\0';char_counter++)
			{
				gretval = isdigit(value[char_counter]);
				if (gretval == 0)
					break;
			}
			if (gretval != 0)
				gmmtss_port = atoi(value);
			else
				gmmtss_port = MMTSS_PORT;

		}

		else if ( 0 == strnicmp(key, "delay", strlen("delay")) )  {
			for ( char_counter=0; value[char_counter] != '\0';char_counter++)
			{
				gretval = isdigit(value[char_counter]);
				if (gretval == 0)
					break;
			}
			if (gretval != 0){
				gdelay = atoi(value);
			}
			else{
				gdelay = SLEEP_TIMER;	
			}
		}
		else if ( 0 == strnicmp(key, "default_capture_filter",
                 strlen("default_capture_filter")) )
		{
			gdefault_filter = (gchar *)g_malloc(strlen(value) + FOR_NULLCHAR);
			if (NULL == gdefault_filter)
			{
				mmt_log("Error:Couldn't read key from configuration file.");
				return FAIL;
			}

			strncpy(gdefault_filter , value, strlen(value));
			gdefault_filter[strlen(value)] = '\0';
		} 
		/**** Byte Alignment Changes Start****/
		else if (0 == strnicmp(key, "is_machine_64bit", strlen("is_machine_64bit")))
		{
			/**Flag to know if the machine on 
			**which MMT runs is 64 bit or 32 bit**/
			for ( char_counter=0;value[char_counter] != '\0';char_counter++)
			{
				gretval = isdigit(value[char_counter]);
				if (gretval == 0)
					break;
			}
			if (gretval != 0)
				isDataFr64 = atoi(value);
			else
				isDataFr64 = BIT32;
		}
		/**** Byte Alignment Changes End****/
		else
			fprintf(fp_log,"Unrecognized key %s obtained in configuration file. Discarding it.\n",key);
		
	}
	fclose(read_config_file_pointer);	
	if ( 1 != ip_flag )
	{		
		ipaddr = get_local_ip();
		g_snprintf(gmmt_ipaddress, IP_ADDR_MAX_STR_LEN, ipaddr );	
	}	
	/* PMC MM#445 */
	if ( gmmtss_ipaddress[0] == '\0' )
		g_snprintf(gmmtss_ipaddress, IP_ADDR_MAX_STR_LEN, DEF_MMTSS_IP_ADDR );

	return PASS;  
} 

/* ****************************************************************************
* Function Name   : mmt_read_config_files
*
* Description	  : This function opens the log file to write and read 
*					the configuration files(att_config.txt and config.txt).
*
* Return		  : int(PASS or FAIL)
******************************************************************************/
int 
mmt_read_config_files ()
{		
	char * file_name = NULL;
	

	if ( NULL == wiresharkLib )
	{
		wiresharkLib =  LoadLibrary(TEXT("libwireshark.dll"));
		if( NULL == wiresharkLib )
		{
			printf("\nError: conn_mgmt.c:mmt_read_config_files() \
				   Error in loading library");	
			return FAIL;	
		}
	}
	full_path = (char**) GetProcAddress(wiresharkLib, "full_path_name");	
	file_name = (char *)calloc( strlen(*full_path) 
								+ strlen(MMT_LOG_FILE)
								+ FOR_NULLCHAR, sizeof(char)
								);
	if ( NULL == file_name )
	{
		printf("\nError: conn_mgmt.c:mmt_read_config_files() \
			   Error in memory allocation");		
		return FAIL;		
	}
	sprintf( file_name, "%s%s", *full_path, MMT_LOG_FILE );
	fp_log = fopen ( file_name, "w+" );	
	
	free(file_name);
	file_name = NULL;	
	if (fp_log  == NULL)
	{	
		printf("Error: conn_mgmt.c:mmt_read_config_files() \
			   Error in creating mmt log file");			
		return FAIL;	
	}
	/* Going to read configuration file config.txt */
	if ( FAIL == read_config_file() )
		mmt_log("Error in reading from config file. \
				Proceeding with default values for config parameters.");
	
	/* Going to read auto_flag_config.txt file for Auto Capture mode */
	get_auto_config_flag();
	atexit(mmt_cleanup);

	return TRUE;
}

/* ****************************************************************************
* Function name   : mmt_open_sockets
*
* Description	  : This function opens the MMT sockets.
*
* Return type	  : int(PASS or FAIL)
******************************************************************************/	
int 
mmt_open_sockets ()
{
	unsigned long non_blocking_recv_option = 1; /* set the receive option as 
												** non blocking 
												*/
	stKeepAlive gkeepAlive_capture_on; /* KeepAlive(Capture_flag_on) message */
	stKeepAlive gkeepAlive_capture_off; /* KeepAlive(Capture_flag_off) 
										** message 
										*/


	gmmt_sock_addr_msg_monitor.sin_family = AF_INET;	

	gmmt_sock_addr_msg_monitor.sin_addr.s_addr = INADDR_ANY;

	gmmt_sock_addr_msg_monitor.sin_port =
		htons(
		(gmmt_port > MIN_VALID_PORT_VALUE && gmmt_port < MAX_VALID_PORT_VALUE)
		? gmmt_port 
		: MESSAGE_MONITORING_PORT
		);

	gmmt_sock_addr_conn_mgmt.sin_family = AF_INET;

	gmmt_sock_addr_conn_mgmt.sin_addr.s_addr = INADDR_ANY;	

	gmmt_sock_addr_conn_mgmt.sin_port =
		htons(
		( gmmt_connmgmt_port > MIN_VALID_PORT_VALUE 
			&& gmmt_connmgmt_port < MAX_VALID_PORT_VALUE )
		? gmmt_connmgmt_port
		: CONN_MGMT_PORT
		);	

	gmmtss_sock_addr.sin_family = AF_INET;
	if( NULL == gmmtss_ipaddress ){
		mmt_log("Please specify MMTSS IP address in configuration file.");		
		return FAIL;		
	}
	gmmtss_sock_addr.sin_addr.s_addr =
		(NULL != gmmtss_ipaddress && (-1 != inet_addr(gmmtss_ipaddress)))
		? inet_addr(gmmtss_ipaddress)
		: inet_addr(DEF_MMTSS_IP_ADDR);

	gmmtss_sock_addr.sin_port =
    htons( (gmmtss_port > MIN_VALID_PORT_VALUE 
			&& gmmtss_port < MAX_VALID_PORT_VALUE)
		? gmmtss_port
		: MMTSS_PORT
		);	
	
	
	gmmt_sock_msg_monitor = socket(AF_INET, SOCK_DGRAM,0);  
	if ( gmmt_sock_msg_monitor < 0 )
	{
		g_snprintf( error_string,
					MMT_ERROR_STRING_MAX_SIZE,
					"Could not create the msg monitoring socket for port: %d"
					" error code: %d.",gmmt_port, WSAGetLastError()
					);
		mmt_log(error_string);
		return FAIL;
	}

	gmmt_sock_conn_mgmt = socket(AF_INET, SOCK_DGRAM,0); 
	if ( gmmt_sock_conn_mgmt < 0 )
	{
		g_snprintf( error_string,
					MMT_ERROR_STRING_MAX_SIZE,
					"Could not create the connection mgmt socket for port: %d"
					" error code: %d.",gmmt_connmgmt_port, WSAGetLastError()
					);
		mmt_log(error_string);
		return FAIL;
	}

	gmmtss_sock = socket(AF_INET, SOCK_DGRAM,0); 
	if ( gmmtss_sock < 0 )
	{
		g_snprintf( error_string,
					MMT_ERROR_STRING_MAX_SIZE,
					"Could not create socket for mmtss port: 5003"
					" error code: %d.", WSAGetLastError()
					);
		mmt_log(error_string);
		return FAIL;
	}

	if ( SOCKET_ERROR == bind(gmmt_sock_msg_monitor,
		       (struct sockaddr*)&gmmt_sock_addr_msg_monitor,
				sizeof(gmmt_sock_addr_msg_monitor))
	   )
	{
		g_snprintf( error_string,
					MMT_ERROR_STRING_MAX_SIZE,
					"Bind failed for message monitoring port for \nIP: %s " 
						"and Port: %d with error code: %d\n",
					gmmt_ipaddress, gmmt_port, WSAGetLastError()
				);
		
		mmt_log(error_string);
		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, error_string);
		
		IS_BIND_FAILED = TRUE;
		IS_SOCKET_OPEN = TRUE;
		/*
		 ** MM#103
		     if( FALSE == gusercontrolledcapture )
		    	capture_start_cb(NULL, NULL);
		 */ 
		return FAIL;		
	}
	else
	{
		g_snprintf( error_string,
					MMT_ERROR_STRING_MAX_SIZE,
					"Bind to port %d is successful.",gmmt_port
				);
		mmt_log(error_string);
	}
	if ( SOCKET_ERROR == bind(gmmt_sock_conn_mgmt,
				(struct sockaddr*)&gmmt_sock_addr_conn_mgmt,
				sizeof(gmmt_sock_addr_conn_mgmt))
				)
	{
		g_snprintf(error_string,
					MMT_ERROR_STRING_MAX_SIZE,
					"Bind failed for connection management port for \nIP: %s " 
						"and Port: %d with error code: %d\n",
					gmmt_ipaddress, gmmt_connmgmt_port, WSAGetLastError()
					);

		mmt_log(error_string);		
		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, error_string);
		
		IS_BIND_FAILED = TRUE;
		IS_SOCKET_OPEN = TRUE;
		/*
		 ** MM#103
			if( FALSE == gusercontrolledcapture )
				capture_start_cb(NULL, NULL);
		 */  
		return FAIL;		
	}
	else
	{
		g_snprintf( error_string,
					MMT_ERROR_STRING_MAX_SIZE,
					"Bind to connection mgmt port %d is successful.",gmmt_connmgmt_port
				);
		mmt_log(error_string);
	}
	
	ioctlsocket(gmmt_sock_conn_mgmt,FIONBIO,&non_blocking_recv_option);		

	/*
	 ** Initialize the size variables for the socket address structures
	 ** of each of the three ports
	 */
	 
	 /* MMT IP Address and Message Monitoring Port */
	gmmt_sock_addr_msg_monitor_size = sizeof(gmmt_sock_addr_msg_monitor);

    /* MMT IP Address and Connection Mgmt Port */
	gmmt_sock_addr_conn_mgmt_size = sizeof(gmmt_sock_addr_conn_mgmt);

    /* MMTSS IP Address and Port */
	gmmtss_sock_addr_size = sizeof(gmmtss_sock_addr);

	/* KeepAlive(Capture_flag_on) message */  
	gkeepAlive_capture_on.message_type = htons(KEEPALIVE_MSG_ID);
	gkeepAlive_capture_on.capture_flag = CAPTURE_FLAG_ON;
	memcpy(gsend_message_capture_on,&gkeepAlive_capture_on,sizeof(stKeepAlive));

	/* KeepAlive(Capture_flag_off) message */  
	gkeepAlive_capture_off.message_type = htons(KEEPALIVE_MSG_ID);
	gkeepAlive_capture_off.capture_flag = CAPTURE_FLAG_OFF;
	memcpy(gsend_message_capture_off,&gkeepAlive_capture_off,sizeof(stKeepAlive));
		
	IS_SOCKET_OPEN = TRUE;
	IS_BIND_FAILED = FALSE;/*MM#103*/
	return TRUE;
}

/******************************************************************************
* Function Name	 : mmt_close_sockets
*
* Description	 : This function close the sockets.
*
* Return		 : void
******************************************************************************/
void
mmt_close_sockets ()
{	
	mmt_log("Closing Connection Management Socket...");
	if ( -1 == closesocket(gmmt_sock_conn_mgmt) )	
		mmt_log("Connection Management Socket is already closed.");
		
	mmt_log("Closing Message Monitoring Socket...");
	if ( -1 == closesocket(gmmt_sock_msg_monitor) )		
		mmt_log("Message Monitoring Socket is already closed.");
	
	mmt_log("Closing MMTSS Socket...");
	if ( -1 == closesocket(gmmtss_sock) )			
		mmt_log("MMTSS Socket is already closed.");
	
	gmmt_sock_conn_mgmt = 0;
	gmmt_sock_msg_monitor = 0;
	gmmtss_sock = 0;
	if ( timeout_keepalive_routine >0 )
	{
		  g_source_remove (timeout_keepalive_routine);
		  timeout_keepalive_routine = 0;	
	}
	if ( timeout_hello_routine >0 )
	{
		g_source_remove (timeout_hello_routine);	
		timeout_hello_routine = 0;
	}
	first_hello_message_received = FALSE;
	colorize_filter_te_as_invalid(pmc_conn_status_indicator);
	/* PMC MM#173 */
	reset_first_pkt_flags();
	IS_SOCKET_OPEN = FALSE;

	// SPR#2964
	gusercontrolledcapture = FALSE;
}

/*************************************************************************************
* Function Name	 : mmt_cleanup
*
* Description	 : This function closes the log file.
*
* Return		 : None
*************************************************************************************/
void 
mmt_cleanup(void)
{	
	write_config_file();
	/* PMC MM#461 - start */
	if ( 1 == chage_mmtss_ip_address_flag )
		update_mmtss_ip_address_config_file();
	/* PMC MM#461 - end */
	if (gdefault_filter)
		g_free(gdefault_filter);
	
	mmt_log("Closing Connection Management Socket...");
	shutdown(gmmt_sock_conn_mgmt,2);	
	if ( -1 == closesocket(gmmt_sock_conn_mgmt) )	
		mmt_log("Connection Management Socket is already closed.");

	mmt_log("Closing Message Monitoring Socket...");
	shutdown(gmmt_sock_msg_monitor,2);	
	if ( -1 == closesocket(gmmt_sock_msg_monitor) )		
		mmt_log("Message Monitoring Socket is already closed.");

	mmt_log("Closing MMTSS Socket...");
	shutdown(gmmtss_sock,2);
	if ( -1 == closesocket(gmmtss_sock) )			
		mmt_log("MMTSS Socket is already closed.");

	gmmt_sock_conn_mgmt = 0;
	gmmt_sock_msg_monitor = 0;
	gmmtss_sock = 0;

	if ( timeout_keepalive_routine >0 )
	{
		  g_source_remove (timeout_keepalive_routine);
		  timeout_keepalive_routine = 0;	
	}
	if ( timeout_hello_routine >0 )
	{
		g_source_remove (timeout_hello_routine);	
		timeout_hello_routine = 0;
	}

	IS_SOCKET_OPEN = FALSE;
	
	if ( NULL != fp_log )
	{
		fclose(fp_log);	
		fp_log = NULL;
	}	
} 

/******************************************************************************
* Function Name  : write_config_file
*
* Description    : This function updates the value of configurable parameters
*                  on the config file.
*
* Return         : None
******************************************************************************/
void 
write_config_file()
{

	FILE* write_config_file_pointer = NULL; /* Config File Pointer */	
	char * file_name = NULL;
	

	if ( NULL == wiresharkLib )
	{
		wiresharkLib =  LoadLibrary(TEXT("libwireshark.dll"));
		if( NULL == wiresharkLib )
		{
			printf("Error: conn_mgmt.c:write_config_file() \
				   Error in loading library");		
			return;		
		}
	}
	full_path = (char**) GetProcAddress(wiresharkLib, "full_path_name");
	file_name = (char *)calloc( strlen(*full_path) 
								+ strlen(AUTO_CONFIG_FILE)
								+ FOR_NULLCHAR, sizeof(char) );
	if ( NULL == file_name )
	{
		mmt_log("Error(conn_mgmt.c:write_config_file): Memory allocation failed.");		
		return ;
	}
	sprintf( file_name, "%s%s", *full_path, AUTO_CONFIG_FILE );
	write_config_file_pointer  = fopen(file_name,"w");

	free(file_name);
	file_name = NULL;
	
	if (NULL == write_config_file_pointer)
	{	
		mmt_log("Error: Couldn't open configuration file for writing.");		
		return;
	}	
	fprintf(write_config_file_pointer,"Autocapture=%d",gautocapture);

	fclose(write_config_file_pointer);  
	write_config_file_pointer = NULL;
}       



/******************************************************************************
* Function Name  : keepalive_routine
*
* Description    : This function sends keepAlive messages to MMTSS
*                  after every 10 seconds.
*
* Return		 : void
******************************************************************************/
void 
keepalive_routine()
{  
	time_t curr; /*variables to store current time*/
	struct tm *today;

	/*
	** Send the keep alive message with capture flag "ON" if user has initiated
	** the capture manually and hasn't stopped it yet or if autocapture mode is
	** set and MMT-MMTSS connection is down. This message shall indicate to
	** MMTSS that MMT is ready to capture packets and MMTSS should start
	** sending packets to MMT.
	*/
	if (gusercontrolledcapture || (gautocapture == AUTOCAPTURE_SET
         && first_hello_message_received == FALSE)) {
		if (1 == KEEPALIVE_SEND_FLAG) /*MM# 103 Start*/
		{	/*MM# 103 End*/
			if(SOCKET_ERROR == sendto(gmmtss_sock, gsend_message_capture_on
				,sizeof(gsend_message_capture_on),0
			 ,(struct sockaddr*)&gmmtss_sock_addr,gmmtss_sock_addr_size)){
				g_snprintf(error_string, MMT_ERROR_STRING_MAX_SIZE
				,"Sending of Keepalive message to MMTSS Failed for MMTSS IP: %s and Port: %d with error code: %d"
				,gmmtss_ipaddress, gmmtss_port, WSAGetLastError());

				/* PMC MM#135 -- Due to "fflush" used in "mmt_log" function, 
				** keepalive timer was getting expired, so putted the code here. 			
				*/
				time(&curr); /* create a "timestamp" */
				today = localtime(&curr);	
				if ( NULL != fp_log )
					fprintf(fp_log, "%02u:%02u:%02u %8s\n", today->tm_hour,
					today->tm_min, today->tm_sec,error_string);	}
		}
		else /*MM# 103 Start */
		{
			if(SOCKET_ERROR == sendto(gmmtss_sock, gsend_message_capture_off
            ,sizeof(gsend_message_capture_on),0
            ,(struct sockaddr*)&gmmtss_sock_addr,gmmtss_sock_addr_size)){
			g_snprintf(error_string, MMT_ERROR_STRING_MAX_SIZE
            ,"Sending of Keepalive message to MMTSS Failed for MMTSS IP: %s and Port: %d with error code: %d"
            ,gmmtss_ipaddress, gmmtss_port, WSAGetLastError());
			
			time(&curr); /* create a "timestamp" */
			today = localtime(&curr);	
			if ( NULL != fp_log )
				fprintf(fp_log, "%02u:%02u:%02u %8s\n", today->tm_hour,
				today->tm_min, today->tm_sec,error_string);	}
		}	/*MM# 103 End*/
	}
	else {
		/*
		** Send the keep alive message with capture flag "OFF" if capture is
		** not in progress currently and autocapture mode is not set or
		** autocapture mode is set and MMT-MMTSS connection is UP because auto
		** -capture mode must come into picture only in case of connection
		** reset between MMT-MMTSS. This message shall indicate to MMTSS that
		** MMT is not ready to capture packets and MMTSS should stop sending
		** packets to MMT
		*/
		
		if(SOCKET_ERROR == sendto(gmmtss_sock, gsend_message_capture_off
           , sizeof(gsend_message_capture_off),0
           ,(struct sockaddr*)&gmmtss_sock_addr,gmmtss_sock_addr_size)){
			g_snprintf(error_string, MMT_ERROR_STRING_MAX_SIZE,
            "Sending of Keepalive message to MMTSS Failed for MMTSS IP: %s and Port: %d with error code: %d"
            ,gmmtss_ipaddress, gmmtss_port, WSAGetLastError());

			/* PMC MM#135 -- Due to "fflush" used in "mmt_log" function, 
			** keepalive timer was getting expired, so putted the code here. 			
			*/
			time(&curr); /* create a "timestamp" */
			today = localtime(&curr);	
			if ( NULL != fp_log )
				fprintf(fp_log, "%02u:%02u:%02u %8s\n", today->tm_hour,
				today->tm_min, today->tm_sec,error_string);										
		}
	}		
}

/**********************************************************************
* Function name   : hello_routine
*
* Description     : This function wait for hello messages from
*                   MMTSS and updates Connection Status on GUI.
*
* Return type     : void
***********************************************************************/
void 
hello_routine()
{
	
	/*SPR 64: Logic to update connection management indicator corrected*/
	if (SOCKET_ERROR != recv(gmmt_sock_conn_mgmt,g_buffer_for_hello_msg,
		sizeof(g_buffer_for_hello_msg), 0)) {
			/*
			** Check the message type field in the received message
			** to verify that HELLO message has been received
			*/
			if (((stHello *)((void *)g_buffer_for_hello_msg))->message_type == htons(HELLO_MSG_ID)) {
				/*  Update the global variable containing the time of the
				** last received hello message
			 */
				time(&g_time_last_hello_recvd_at);
				KEEPALIVE_SEND_FLAG=1;  /* MM#103 */
				/*
				** Update the global variable indicating that a hello message has
				** been received correctly from MMTSS and hence implying that MMT-
				** MMTSS interface may be up after checking that message
				** monitoring port is up as well
			 */
				if (!first_hello_message_received) 
				{
					if ( (SOCKET_ERROR != bind( 
								g_temp_socket,
								(struct sockaddr*)&gmmt_sock_addr_msg_monitor,
								sizeof(gmmt_sock_addr_msg_monitor)) )
								|| (WSAEADDRINUSE == WSAGetLastError())
						) 
					{
							mmt_log("The Message monitoring port is not open \
									 at MMT, cannot start capturing.");
							first_hello_message_received = FALSE;
							colorize_filter_te_as_invalid(pmc_conn_status_indicator);																	
					}
					else{	
						/* MM#171 - start */
						mmt_log("First hello msg received.");
						keepalive_routine();						
						/* MM#171 - end */

						/* PMC MM#173 */
						reset_first_pkt_flags();
						/*****Sequence ID #1204*****/
						first_hello_message_received = TRUE;
						/* PMC MM_13_92 */
						MMTSS_VERSION_FLAG = TRUE;
						colorize_filter_te_as_valid(pmc_conn_status_indicator);
						/* MM#171 */
						//keepalive_routine();
						
						/* SPR#3569 - start */	
						if( FALSE == gusercontrolledcapture
							&& gautocapture != AUTOCAPTURE_SET 
							)
							mmt_close_sockets();
						/* SPR#3569 - end */	

						/*
						** SPR 63: start capturing automatically if autocapture
						** mode is on and capture is not in progress already
					 */
						if( FALSE == gusercontrolledcapture
							&& gautocapture == AUTOCAPTURE_SET 
							)
							capture_start_cb(NULL, NULL);

						/*SPR 63*/
					}
				}
				/*SPR 64*/
			}
	}
	else {
		/*
		** if maximum time has elapsed since last HELLO message was received,
		** trurn the connection indicator off indicating the down status of
		** MMT-MMTSS interface
		*/
		if (difftime(time(NULL)
			,g_time_last_hello_recvd_at) >= MAX_TIMER_VAL_FR_HELLO_MSG) {

				/* PMC MM#173 */
				if ( TRUE == first_hello_message_received )				
					reset_first_pkt_flags();
				
				first_hello_message_received = FALSE;
				KEEPALIVE_SEND_FLAG=0;/* MM# 103*/
				g_snprintf( error_string,
							MMT_ERROR_STRING_MAX_SIZE,
							"MMT-MMTSS connection is probably down."
							"No hello message received on MMT at IP: %s and" 
							"Conn Mgmt Port: %d with error code: %d",
							gmmt_ipaddress,
							gmmt_connmgmt_port, 
							WSAGetLastError()
					);
				mmt_log(error_string);
				/*
				** update the global variable containing the time last hello
				** message was received with current system time.This is 
				** required otherwise the difference of the last received 
				** message and current time shall alwaz surpass maximum 
				** time once it overshoots it
				*/
				time(&g_time_last_hello_recvd_at);
				colorize_filter_te_as_invalid(pmc_conn_status_indicator);
				/* PMC MM#173 */
				reset_first_pkt_flags();
		}
	}		
}

/**********************************************************************
* Function name   : mmt_conn_mgmt_initialize
*
* Description     : This function initialize connection management. 
*
* Return type     : int(TRUE/FALSE)
***********************************************************************/
int 
mmt_conn_mgmt_initialize ()
{	 		  

	IS_BIND_FAILED = FALSE;
	mmt_open_sockets();

	if ( FALSE == IS_BIND_FAILED )
	{
		keepalive_routine();	 
		/* MM#171 */
		mmt_log("First keepalive msg sent.");

		timeout_keepalive_routine = g_timeout_add(gdelay*1000, 
			(GtkFunction)keepalive_routine,
			(gpointer)NULL
			);
		if ( timeout_keepalive_routine >0 )
			mmt_log("Keepalive timer thread started successfully.");
		else
		{
			mmt_log("Error: keepalive timer thread startup failed.");
			return FALSE;
		}

		/* MM#171 - start */

		/*timeout_hello_routine = g_timeout_add(gdelay*200, 
			(GtkFunction)hello_routine,
			(gpointer)NULL
			);*/
		timeout_hello_routine = g_timeout_add(150, 
			(GtkFunction)hello_routine,
			(gpointer)NULL
			);	
		/* MM#171 - end */
		if ( timeout_hello_routine >0 )
			mmt_log("Hello timer(150 msecs) thread started successfully.");
		else
		{
			mmt_log("Error: hello timer(150 msecs) thread startup failed.");
			return FALSE;
		}
		return TRUE;
	}
	else
		return FALSE;
}

/**********************************************************************
* Function name   : get_auto_config_flag
*
* Description     : This function wait for hello messages from
*                   MMTSS and updates Connection Status on GUI.
*
* Return type     : void
***********************************************************************/
void 
get_auto_config_flag ()
{
	FILE*  read_auto_config_file_pointer = NULL; /*config file pointer*/
	guchar *key = NULL; /* tokens to read in the key */
	guchar *value = NULL; /* token value from the config file entry at each line*/
	char * file_name = NULL;
	register guint32	char_counter;
	guint32 gretval; 
	gchar *line = (gchar *)ep_alloc(MAX_CONFIG_LINE_SIZE + FOR_NULLCHAR);
	*line = '\0'; /*initialize line string*/

	if ( NULL == wiresharkLib )
	{
		wiresharkLib =  LoadLibrary(TEXT("libwireshark.dll"));
		if( NULL == wiresharkLib )
		{
			printf("\nconn_mgmt.c:mmt_initialize() Error in loading library");			
			return;
		}
	}
	full_path = (char**) GetProcAddress(wiresharkLib, "full_path_name");
	file_name = (char *)calloc( strlen(*full_path) 
								+ strlen(AUTO_CONFIG_FILE)
								+ FOR_NULLCHAR, sizeof(char) 
							  );
	if ( NULL == file_name )	
		return;
	
	sprintf( file_name, "%s%s", *full_path, AUTO_CONFIG_FILE );
	read_auto_config_file_pointer  = fopen(file_name,"r");

	free(file_name);
	file_name = NULL;

	if (read_auto_config_file_pointer  == NULL)		
		return;
		
	while ( NULL != fgets ( line, 
							MAX_CONFIG_LINE_SIZE, 
							read_auto_config_file_pointer
							)) 
	{    
		if( is_blank_line(line) )			 				
			continue;
		key = strtok(line, CONFIG_FILE_KEY_VALUE_SEPARATOR);
		if (NULL == key) 
			return;		
		else 
		{
			value = strtok(NULL, END_OF_LINE_STRING);
			if ( NULL == value)
				return;			
		}
		 
		if( 0 == strnicmp(key, "Autocapture", strlen("Autocapture")) )  
		{
			for ( char_counter=0; value[char_counter] != '\0';char_counter++)
			{
				gretval = isdigit(value[char_counter]);
				if ( 0 == gretval )
					break;
			}
			if ( 0 != gretval )
				gautocapture = atoi(value);							
		}
	}
	fclose(read_auto_config_file_pointer);
	read_auto_config_file_pointer = NULL;
} 

/* PMC MM#461 - start */
/**********************************************************************
* Function name   : update_mmtss_ip_address_config_file
*
* Description     : This function update mmtss ip address in config.txt 
*					file at the exit of MMT.
*
* Return type     : void
***********************************************************************/
void
update_mmtss_ip_address_config_file()
{
	FILE* write_config_file_pointer = NULL; /* Config File Pointer */	
	char * file_name = NULL;
	

	if ( NULL == wiresharkLib )
	{
		wiresharkLib =  LoadLibrary(TEXT("libwireshark.dll"));
		if( NULL == wiresharkLib )
		{
			printf("\nError: conn_mgmt.c:write_config_file() \
				   Error in loading library");		
			return;		
		}
	}
	full_path = (char**) GetProcAddress(wiresharkLib, "full_path_name");
	file_name = (char *)calloc( strlen(*full_path) 
								+ strlen(CONFIG_FILE)
								+ FOR_NULLCHAR, sizeof(char) );
	if ( NULL == file_name )
	{
		mmt_log("Error(conn_mgmt.c:write_config_file): Memory allocation failed.");		
		return ;
	}
	sprintf( file_name, "%s%s", *full_path, CONFIG_FILE );
	write_config_file_pointer  = fopen(file_name,"w");

	free(file_name);
	file_name = NULL;
	
	if (NULL == write_config_file_pointer)
	{	
		mmt_log("Error: Couldn't open configuration file for writing.");		
		return;
	}
	if ( 1 == ip_flag )
		fprintf(write_config_file_pointer,"mmt_ip_address=%s\n",gmmt_ipaddress);
		
	fprintf(write_config_file_pointer,"mmtss_ip_address=%s\n",gmmtss_ipaddress);
	fprintf(write_config_file_pointer,"mmt_msg_monitor_port=%d\n",gmmt_port);
	fprintf(write_config_file_pointer,"mmt_conn_mgmt_port=%d\n",gmmt_connmgmt_port);	
	fprintf(write_config_file_pointer,"mmtss_port=%d\n",gmmtss_port);
	fprintf(write_config_file_pointer,"delay=%d\n",gdelay);
	/**** Byte Alignment Changes Start****/
	fprintf(write_config_file_pointer,"is_machine_64bit=%d",isDataFr64);
	/**** Byte Alignment Changes End****/
	fclose(write_config_file_pointer);  
	write_config_file_pointer = NULL;
}

/**********************************************************************
* Function name   : update_mmtss_ip_address
*
* Description     : This function update mmtss ip address.
*
* Return type     : void
***********************************************************************/
void 
update_mmtss_ip_address ( const char *ip )
{
	if ( NULL == ip )
		return;

	memset(gmmtss_ipaddress, 0, sizeof(gmmtss_ipaddress));
	g_snprintf(gmmtss_ipaddress, IP_ADDR_MAX_STR_LEN, ip );
	chage_mmtss_ip_address_flag = 1;
	/* SPR#3676 - start */
	if ( TRUE == gusercontrolledcapture )
		simple_dialog(ESD_TYPE_INFO, ESD_BTN_OK,"Changes will be effective only when new MMT capture starts");
	/* SPR#3676 - end */
}
/* PMC MM#461 - end */

/* MM#171 - start */

/**********************************************************************
* Function name   : add_new_hello_timer
*
* Description     : This function remove hello msg timer for 150 msecs
*					and	add new hello msg timer for 500 msecs.	
*
* Return type     : void
***********************************************************************/
void 
add_new_hello_timer()
{
	if ( timeout_hello_routine >0 )
	{
		if (TRUE == g_source_remove (timeout_hello_routine))	
		{	
			mmt_log("Hello timer(150 msecs) thread removed successfully.");
			timeout_hello_routine = 0;
		}
	}
	timeout_hello_routine = g_timeout_add(500, 
		(GtkFunction)hello_routine,
		(gpointer)NULL
		);
	if ( timeout_hello_routine >0 )
		mmt_log("Hello timer(500 msecs) thread started successfully.");
	else			
		mmt_log("Error: hello timer(500 msecs) thread startup failed.");	
}
/* MM#171 - end */