/* conn_mgmt.h
 *
 * $Id: conn_mgmt.h 24577 2008-03-06 18:26:09Z stig $
 * For connection management: keealive_routine(),hello_routine(),read_config_file(),write_config_file(),mmt_initialize(),mmt_cleanup()
 * Wireshark - Network traffic analyzer
 * By Aricent
 * Copyright 1998 Gerald Combs
 *
 * Richard Sharpe, 13-Feb-1999, added support for initializing structures
 *                              needed by dissect routines
 * Jeff Foster,    2001/03/12,  added support tabbed hex display windowss
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#pragma comment(lib, "Ws2_32.lib")

#ifndef CONN_MGMT_H
#define CONN_MGMT_H

#include <gtk/gtk.h>
#include <winsock2.h>
#include <ws2tcpip.h>

#define PASS								1
#define FAIL								0
#define CONN_MGMT_PORT						5004	/* MMT Connection Management Port */
#define MMTSS_PORT							5003
#define MESSAGE_MONITORING_PORT				5002
#define KEEPALIVE_MSG_ID					0x000c
#define HELLO_MSG_ID						0x000c
#define CAPTURE_FLAG_ON						'1'
#define CAPTURE_FLAG_OFF					'0'
#define MAX_CONFIG_LINE_SIZE				128
/* PMC Support for absolute path to open file. IV-Sheet#253 - start */
//#define CONFIG_FILE						    "./config.txt"
#define CONFIG_FILE						    "\\config.txt"
#define MMT_LOG_FILE						 "\\mmt.log"
#define AUTO_CONFIG_FILE					"\\auto_flag_config.txt"
char ** full_path;
/* PMC Support for absolute path to open file. IV-Sheet#253 - end */
/* PMC Autodetect LocalHost IP start */
int ip_flag;
/* PMC Autodetect LocalHost IP end */
#define FOR_NULLCHAR						 1
#define AUTOCAPTURE_SET						 1
#define MAX_TIMER_VAL_FR_HELLO_MSG			 30.00	/*seconds*/
#define SLEEP_TIMER							 10	/*seconds*/

/* PMC Autodetect LocalHost IP Start */
#define IP_ADDRLEN  16
#define MAX_NODE_LENGTH  127
#define SUCCESS 0
#define FAILURE 1
/* PMC Autodetect LocalHost IP End */

/**** Byte Alignment Changes Start****/
/**Flag to know if the machine on 
 **which MMT runs is 64 bit or 32 bit**/

#define	BIT32	0
#define BIT64	1	

__declspec(dllexport) int isDataFr64;

/**** Byte Alignment Changes End****/

/*
 ** This Message is sent to MMTSS,where capture_flag
 ** indicate whether MMTSS should initiate/continue
 ** capturing and forwarding messages to MMT or it
 ** should stop capturing
 */

typedef struct {						
	guint16  message_type;
	guchar   capture_flag;
 } stKeepAlive;

/*
 ** This Message is sent to MMT by MMTSS on receiving
 ** First Keepalive. It follows the MMTSS header structure
 */
typedef struct {
	guint8	fap_ne;
	guint8	inf_grpid;
	guint16 message_type;
	guint16 seq_id;
	guint16 length;	
	guint32 time_stamp; 
	guint32 time_stamp_ms; 
	guint8  sender_id;
	guint8  dest_id;
	guint8  frag_num;
	guint8  frag_count;
 } stHello;

/* PMC MM_13_92 */
int MMTSS_VERSION_FLAG;

void write_config_file();

/*
 ** The delay between two outgoing keep alive messages.
 ** Its value is read from configuration file
 */
guint32					gdelay;
/*Default autocapture mode.Its value is read from configuration file*/
gboolean					gautocapture;

/*
 ** Widget to refer to connection indicator in MMT gui. This widget
 ** is accesssed and updated from hello_routine() in conn_mgmt.c file
 ** and is added to MMT gui from create_main_window() function in main.c
 */
GtkWidget				*pmc_conn_status_indicator; 		


/*
 ** Widget to refer to auotcapture button on MMT toolbar in MMT gui. This
 ** widget is added to MMT Toolbar from create_main_window() function in main.c.
 ** It is toggled from autocapture_mode_changed() function in menu.c
 ** andits current status is queried from
 ** mmt_toolbar_toggle_autocapture_mode_cb() function in menu.c
 */
#if GTK_CHECK_VERSION(2,4,0)
	GtkToolItem	*pmc_autocapture_button;
#else
	GtkWidget *pmc_autocapture_button;
#endif


/*
 ** This variable is defined in conn_mgmt.c and is used to identify the
 ** status of the MMT-MMTSS connection
 */
extern gboolean			first_hello_message_received;

/*
 ** This variable acts as a flag to identify when user has manually started
 ** the capturing. Is required to identify autopcaturing from user started
 ** mode of capturing. is modified in capture_dlg.c on clicking of
 ** start/stop capture button by the user. Is accessed in conn_mgmt.c
 ** by keepalive_routine and hello_routine and in menu.c by autocapture
 ** mode toggle routines
 */
gboolean				gusercontrolledcapture;

/*
 ** gdefault_filter variable shall contain the default filter to be appended
 ** to all the capture filters automatically.It is read from the configuration
 ** file at the startup in read_config_file() function and is accessed in
 ** capture_dlg_prep function in capture_dlg.c file
 */
guchar					*gdefault_filter;

/* writes message into the log file.*/ 
void mmt_log(char *message);


/* reads the configuration file.*/ 
int read_config_file();

/* 
 ** Initializes MMT and reads the values of configuration parameters 
 ** from the config file. It also initializes all the sockets required by MMT.
 */ 
/* PMC I&V - 199 - start */

/* void mmt_initialize(); */
/* int mmt_initialize(); */

/* closing sockets when capture will stop */
void mmt_close_sockets();

/* funtion to open and bind MMT sockets */
int mmt_open_sockets();

/* funtion to read auto_flag_config.txt file */
void get_auto_config_flag();

/* funtion to read auto_flag_config.txt, config.txt 
** and openinig mmt.log file. 
*/
int mmt_read_config_files();

/* function initialize connection management. */
int mmt_conn_mgmt_initialize();

/* flag to check Socket open */
int IS_SOCKET_OPEN;

/* flag to check bind failed */
int IS_BIND_FAILED;

/* function to check blank lines in the configuration file */
int is_blank_line(char *line);

/* variable for keepalive message timer */
guint timeout_keepalive_routine;

/* variable for hello message timer */
guint timeout_hello_routine;

/* PMC I&V - 199 - end */

/* closes the log file.*/ 
void mmt_cleanup(void);

/* Updates the value of configurable parameters in the configuration file.*/ 
void write_config_file();

/* sends keepAlive messages to MMTSS after every defined number of seconds.*/
void keepalive_routine(); 

/* wait for hello messages from MMTSS and updates Connection Status on GUI*/
void hello_routine();

/* PMC Autodetect LocalHost IP Start */

/* Convert any Numeric IP Address to Text String */
const char *inet_ntop2(int af, const void *src, char *dst, socklen_t cnt);

/* Get IP address of Local Host */
char *get_local_ip();
/* PMC Autodetect LocalHost IP End */

/* PMC MM#461 - start */

/* flag to check to rewrite the mmtss ip address in config.txt file */
int chage_mmtss_ip_address_flag;

/* This function will reset the global variable gmmtss_ipaddress with ip */
void update_mmtss_ip_address ( const char *ip );

/* This function will update mmtss ip address in the config file(config.txt) */
void update_mmtss_ip_address_config_file();

/* PMC MM#461 - end */

/* MM#171 */
void add_new_hello_timer();

/* MM#103 */
unsigned int KEEPALIVE_SEND_FLAG;

#endif //#ifndef CONN_MGMT_H

